package com.hand.hls.web.svc.components;

import com.alibaba.fastjson.JSON;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.impl.RequestHelper;
import com.hand.hap.core.web.view.XMap;
import com.hand.hap.core.web.view.XMapHandler;
import com.hand.hap.core.web.view.XMapParser;
import com.hand.hls.web.svc.ServiceBuilder;
import com.hand.hls.web.svc.ServiceContext;
import com.hand.hls.web.svc.ServiceTagFactory;
import com.hand.hls.web.svc.domain.Model;
import com.hand.hls.web.svc.domain.Operations;
import com.hand.hls.web.svc.utils.ModelDeleteParser;
import com.hand.hls.web.svc.utils.ModelQueryParser;
import com.hand.hls.web.svc.utils.ModelSaveParser;
import com.hand.hls.web.svc.utils.ModelUpdateParser;
import freemarker.template.Configuration;
import freemarker.template.Template;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSourceUtils;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.support.RequestContextUtils;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.sql.PreparedStatement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created with IntelliJ IDEA.
 * User: yang
 * Date: 2017-12-05
 * Time: 9:34
 */
@Component
public class BmOperation implements ApplicationContextAware{
    private PathMatchingResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();

    @Autowired
    private NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    @Autowired
    private ServiceTagFactory serviceTagFactory;
    private ApplicationContext applicationContext;

    private Logger logger = LoggerFactory.getLogger(BmOperation.class);

    public String loadBM(String bmPath,HttpServletRequest request) throws Exception {
        try {
            Configuration configuration = FreemarkerConfigHolder.getConfiguration();
            Template template = configuration.getTemplate(bmPath);
            HashMap<String, Object> map = new HashMap<>();
            ServiceContext serviceContext = new ServiceContext(configuration, serviceTagFactory, map, request, null);
            if (template == null) {
                logger.debug("could not find the target resource:{}", bmPath);
                throw new FileNotFoundException(bmPath + " could not found");
            }
            String content;
            try(StringWriter writer = new StringWriter()){
                template.process(new HashMap(0),writer);
                writer.flush();
                content = writer.toString();
            }
            XMap xMap = XMapParser.parseReader(new StringReader(content), new XMapHandler());
            String bm = ServiceBuilder.build(xMap, serviceContext);
            System.out.println(bm);
            return bm;
        } catch (Exception e) {
            logger.error("getBmRealPath error", e);
            throw e;
        }
    }

    public List<Map<String, Object>> query(String bmJson, String paramJson) {
        Model model = JSON.parseObject(bmJson, Model.class);
        Map param = JSON.parseObject(paramJson, Map.class);
        String sql = ModelQueryParser.parseQuery(model);
        HashMap<String, Object> map = prepareParams(param);
        return namedParameterJdbcTemplate.queryForList(sql, map);
    }

    @SuppressWarnings("unchecked")
    public Object insert(String bmJson, String paramJson) {
        Model model = JSON.parseObject(bmJson, Model.class);
        Map param = JSON.parseObject(paramJson, Map.class);
        Map map = prepareParams(param);
        String sql;
        if(CollectionUtils.isEmpty(model.getFields())){
            sql = ModelSaveParser.parseSaveWithParamOnly(model, param);
            map = param;
        }else {
            sql = ModelSaveParser.parseSaveWithParam(model, map);
            map = prepareParams(param);
        }
        KeyHolder keyHolder = new GeneratedKeyHolder();
        MapSqlParameterSource parameterSource = new MapSqlParameterSource(map);
        int update = namedParameterJdbcTemplate.update(sql, parameterSource, keyHolder);
        List<Map<String, Object>> keyList = keyHolder.getKeyList();
        return keyList;
    }

    public Object update(String bmJson, String paramJson) {
        Model model = JSON.parseObject(bmJson, Model.class);
        Map param = JSON.parseObject(paramJson, Map.class);
        Map map = prepareParams(param);
        String sql;
        if(CollectionUtils.isEmpty(model.getFields())){
            sql = ModelUpdateParser.parseUpdateWithParamOnly(model,map,param);
            map = param;
        }else {
            sql = ModelUpdateParser.parseUpdateWithParam(model, map);
            map = prepareParams(param);
        }
        KeyHolder keyHolder = new GeneratedKeyHolder();
        MapSqlParameterSource parameterSource = new MapSqlParameterSource(map);
        int update = namedParameterJdbcTemplate.update(sql, parameterSource, keyHolder);
        List<Map<String, Object>> keyList = keyHolder.getKeyList();
        return keyList;
    }

    public Object delete(String bmJson, String paramJson) {
        Model model = JSON.parseObject(bmJson, Model.class);
        Map param = JSON.parseObject(paramJson, Map.class);
        String sql = ModelDeleteParser.parseDelete(model, param);
        System.out.println(sql);
        HashMap<String, Object> map = prepareParams(param);
        namedParameterJdbcTemplate.execute(sql, map, PreparedStatement::execute);
        return bmJson;
    }

    public Object opQuery(String bmJson, String paramJson){
        Model model = JSON.parseObject(bmJson, Model.class);
        Map param = JSON.parseObject(paramJson, Map.class);
        Operations operations = model.getOperations();
        if(operations == null || StringUtils.isEmpty(operations.getQuery())){
            throw new RuntimeException("could not found operation[query]");
        }
        String sql = operations.getQuery();
        HashMap<String, Object> map = prepareParams(param);
        return namedParameterJdbcTemplate.queryForList(sql, map);
    }

    public Object opInsert(String bmJson, String paramJson) {
        Model model = JSON.parseObject(bmJson, Model.class);
        Map param = JSON.parseObject(paramJson, Map.class);
        HashMap<String, Object> map = prepareParams(param);
        Operations operations = model.getOperations();
        if(operations == null || StringUtils.isEmpty(operations.getInsert())){
            throw new RuntimeException("could not found operation[insert]");
        }
        String sql = operations.getInsert();
        return namedParameterJdbcTemplate.execute(sql, map, PreparedStatement::execute);
    }

    public Object opUpdate(String bmJson, String paramJson) {
        Model model = JSON.parseObject(bmJson, Model.class);
        Map param = JSON.parseObject(paramJson, Map.class);
        HashMap<String, Object> map = prepareParams(param);
        Operations operations = model.getOperations();
        if(operations == null || StringUtils.isEmpty(operations.getUpdate())){
            throw new RuntimeException("could not found operation[update]");
        }
        String sql = operations.getUpdate();
        return namedParameterJdbcTemplate.update(sql, map);
    }

    public Object opDelete(String bmJson, String paramJson) {
        Model model = JSON.parseObject(bmJson, Model.class);
        Map param = JSON.parseObject(paramJson, Map.class);
        HashMap<String, Object> map = prepareParams(param);
        Operations operations = model.getOperations();
        if(operations == null || StringUtils.isEmpty(operations.getDelete())){
            throw new RuntimeException("could not found operation[delete]");
        }
        String sql = operations.getDelete();
        namedParameterJdbcTemplate.execute(sql, map, PreparedStatement::execute);
        return bmJson;
    }

    private HashMap<String, Object> prepareParams(Map param) {
        IRequest currentRequest = RequestHelper.getCurrentRequest();
        Map<String, Object> attributeMap = currentRequest.getAttributeMap();
        HashMap<String, Object> map = new HashMap<>(attributeMap.size() + param.size());
        attributeMap.forEach((key, value) -> map.put("$session." + key, value));
        param.forEach((key, value) -> map.put("$params." + key, value));
        map.put("$session.userId", currentRequest.getUserId());
        map.put("$session.roleId", currentRequest.getRoleId());
        map.put("$session.employeeCode", currentRequest.getEmployeeCode());
        map.put("$session.userName", currentRequest.getUserName());
        map.put("$session.roleIds", currentRequest.getAllRoleId());
        return map;
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }
}
